-module(doudizhu_ai).
-behaviour(gen_server).

-export([
    start_link/1,
    make_decision/2,
    analyze_cards/1,
    update_game_state/2
]).

-export([
    init/1,
    handle_call/3,
    handle_cast/2,
    handle_info/2,
    terminate/2,
    code_change/3
]).

-include("card_types.hrl").

%% API 函数
start_link(PlayerId) ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [PlayerId], []).

make_decision(GameState, Options) ->
    gen_server:call(?MODULE, {make_decision, GameState, Options}).

analyze_cards(Cards) ->
    gen_server:call(?MODULE, {analyze_cards, Cards}).

update_game_state(Event, Data) ->
    gen_server:cast(?MODULE, {update_game_state, Event, Data}).

%% Callback 函数
init([PlayerId]) ->
    {ok, #state{player_id = PlayerId}}.

handle_call({make_decision, GameState, Options}, _From, State) ->
    {Decision, NewState} = calculate_best_move(GameState, Options, State),
    {reply, Decision, NewState};

handle_call({analyze_cards, Cards}, _From, State) ->
    Analysis = perform_cards_analysis(Cards, State),
    {reply, Analysis, State};

handle_call(_Request, _From, State) ->
    {reply, ok, State}.

handle_cast({update_game_state, Event, Data}, State) ->
    NewState = process_game_event(Event, Data, State),
    {noreply, NewState};

handle_cast(_Msg, State) ->
    {noreply, State}.

handle_info(_Info, State) ->
    {noreply, State}.

terminate(_Reason, _State) ->
    ok.

code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

%% 内部函数

% 计算最佳移动
calculate_best_move(GameState, Options, State) ->
    % 使用策略模块计算最佳移动
    Strategy = doudizhu_ai_strategy:choose_strategy(GameState, State),
    Move = doudizhu_ai_strategy:execute_strategy(Strategy, GameState, State),
    {Move, State#state{strategy_cache = #{current_strategy => Strategy}}}.

% 分析手牌
perform_cards_analysis(Cards, State) ->
    % 简化的手牌分析
    HandValue = doudizhu_ai_strategy:analyze_hand_value(Cards),
    {hand_value, HandValue}.

% 处理游戏事件
process_game_event(card_played, {Player, Cards}, State) ->
    % 更新已知牌信息
    NewPlayedCards = [{Player, Cards} | State#state.played_cards],
    State#state{played_cards = NewPlayedCards};

process_game_event(new_hand, Cards, State) ->
    % 设置新的手牌
    State#state{hand_cards = Cards};

process_game_event(role_assigned, Role, State) ->
    % 设置角色（地主或农民）
    State#state{role = Role};

process_game_event(_, _, State) ->
    % 默认情况下不改变状态
    State.